@@ -4,7 +4,9 @@
 
 #include "HTMLEditor.h"
 
+#include "CSSEditUtils.h"
 #include "HTMLEditUtils.h"
+
 #include "mozilla/Attributes.h"
 #include "mozilla/PresShell.h"
 #include "mozilla/PresShellInlines.h"
@@ -58,24 +60,27 @@ static int32_t GetCSSFloatValue(nsComputedDOMStyle* aComputedStyle,
 
   // get the computed CSSValue of the property
   nsAutoCString value;
-  nsresult rv = aComputedStyle->GetPropertyValue(aProperty, value);
-  if (NS_FAILED(rv)) {
-    NS_WARNING("nsComputedDOMStyle::GetPropertyValue() failed");
-    return 0;
-  }
-
+  aComputedStyle->GetPropertyValue(aProperty, value);
   // We only care about resolved values, not a big deal if the element is
   // undisplayed, for example, and the value is "auto" or what not.
+  nsresult rv = NS_OK;
   int32_t val = value.ToInteger(&rv);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "nsAString::ToInteger() failed");
   return NS_SUCCEEDED(rv) ? val : 0;
 }
 
-class ElementDeletionObserver final : public nsStubMutationObserver {
+/******************************************************************************
+ * mozilla::ElementDeletionObserver
+ *****************************************************************************/
+
+class ElementDeletionObserver final : public nsStubMultiMutationObserver {
  public:
   ElementDeletionObserver(nsIContent* aNativeAnonNode,
                           Element* aObservedElement)
-      : mNativeAnonNode(aNativeAnonNode), mObservedElement(aObservedElement) {}
+      : mNativeAnonNode(aNativeAnonNode), mObservedElement(aObservedElement) {
+    AddMutationObserverToNode(aNativeAnonNode);
+    AddMutationObserverToNode(aObservedElement);
+  }
 
   NS_DECL_ISUPPORTS
   NS_DECL_NSIMUTATIONOBSERVER_PARENTCHAINCHANGED
@@ -106,7 +111,7 @@ void ElementDeletionObserver::ParentChainChanged(nsIContent* aContent) {
   NS_RELEASE_THIS();
 }
 
-void ElementDeletionObserver::NodeWillBeDestroyed(const nsINode* aNode) {
+void ElementDeletionObserver::NodeWillBeDestroyed(nsINode* aNode) {
   NS_ASSERTION(aNode == mNativeAnonNode || aNode == mObservedElement,
                "Wrong aNode!");
   if (aNode == mNativeAnonNode) {
@@ -121,6 +126,10 @@ void ElementDeletionObserver::NodeWillBeDestroyed(const nsINode* aNode) {
   NS_RELEASE_THIS();
 }
 
+/******************************************************************************
+ * mozilla::HTMLEditor
+ *****************************************************************************/
+
 ManualNACPtr HTMLEditor::CreateAnonymousElement(nsAtom* aTag,
                                                 nsIContent& aParentContent,
                                                 const nsAString& aAnonClass,
@@ -198,8 +207,6 @@ ManualNACPtr HTMLEditor::CreateAnonymousElement(nsAtom* aTag,
   auto* observer = new ElementDeletionObserver(newNativeAnonymousContent,
                                                aParentContent.AsElement());
   NS_ADDREF(observer);  // NodeWillBeDestroyed releases.
-  aParentContent.AddMutationObserver(observer);
-  newNativeAnonymousContent->AddMutationObserver(observer);
 
 #ifdef DEBUG
   // Editor anonymous content gets passed to PostRecreateFramesFor... which
@@ -350,7 +357,7 @@ nsresult HTMLEditor::RefreshEditingUI() {
   }
 
   // If we're not in a document, don't try to add resizers
-  if (!selectionContainerElement->IsInUncomposedDoc()) {
+  if (!selectionContainerElement->IsInComposedDoc()) {
     return NS_OK;
   }
 
@@ -436,7 +443,7 @@ nsresult HTMLEditor::RefreshEditingUI() {
   }
 
   // now, let's display all contextual UI for good
-  nsIContent* hostContent = GetActiveEditingHost();
+  nsIContent* hostContent = ComputeEditingHost();
 
   if (IsObjectResizerEnabled() && focusElement &&
       HTMLEditUtils::IsSimplyEditableNode(*focusElement) &&
@@ -508,8 +515,7 @@ nsresult HTMLEditor::GetPositionAndDimensions(Element& aElement, int32_t& aX,
                                               int32_t& aMarginLeft,
                                               int32_t& aMarginTop) {
   // Is the element positioned ? let's check the cheap way first...
-  bool isPositioned =
-      aElement.HasAttr(kNameSpaceID_None, nsGkAtoms::_moz_abspos);
+  bool isPositioned = aElement.HasAttr(nsGkAtoms::_moz_abspos);
   if (!isPositioned) {
     // hmmm... the expensive way now...
     nsAutoString positionValue;
@@ -570,7 +576,7 @@ nsresult HTMLEditor::GetPositionAndDimensions(Element& aElement, int32_t& aX,
 nsresult HTMLEditor::SetAnonymousElementPositionWithoutTransaction(
     nsStyledElement& aStyledElement, int32_t aX, int32_t aY) {
   nsresult rv;
-  rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+  rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
       aStyledElement, *nsGkAtoms::left, aX);
   if (rv == NS_ERROR_EDITOR_DESTROYED) {
     NS_WARNING(
@@ -582,7 +588,7 @@ nsresult HTMLEditor::SetAnonymousElementPositionWithoutTransaction(
       NS_SUCCEEDED(rv),
       "CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(nsGkAtoms::left) "
       "failed, but ignored");
-  rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+  rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
       aStyledElement, *nsGkAtoms::top, aY);
   if (rv == NS_ERROR_EDITOR_DESTROYED) {
     NS_WARNING(
