@@ -52,7 +52,7 @@ bool StateUpdatingCommandBase::IsCommandEnabled(Command aCommand,
   if (!htmlEditor) {
     return false;
   }
-  if (!htmlEditor->IsSelectionEditable()) {
+  if (!htmlEditor->IsModifiable() || !htmlEditor->IsSelectionEditable()) {
     return false;
   }
   if (aCommand == Command::FormatAbsolutePosition) {
@@ -89,12 +89,16 @@ nsresult StateUpdatingCommandBase::GetCommandStateParams(
   if (NS_WARN_IF(!htmlEditor)) {
     return NS_ERROR_FAILURE;
   }
-  nsAtom* tagName = GetTagName(aCommand);
+  nsStaticAtom* tagName = GetTagName(aCommand);
   if (NS_WARN_IF(!tagName)) {
     return NS_ERROR_UNEXPECTED;
   }
-  nsresult rv = GetCurrentState(MOZ_KnownLive(tagName),
-                                MOZ_KnownLive(htmlEditor), aParams);
+  // MOZ_KnownLive(htmlEditor) because the lifetime of aEditorBase is guaranteed
+  // by the callers.
+  // MOZ_KnownLive(tagName) because nsStaticAtom instances are alive until
+  // shutting down.
+  nsresult rv = GetCurrentState(MOZ_KnownLive(*tagName),
+                                MOZ_KnownLive(*htmlEditor), aParams);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                        "StateUpdatingCommandBase::GetCurrentState() failed");
   return rv;
@@ -124,10 +128,12 @@ nsresult PasteNoFormattingCommand::DoCommand(Command aCommand,
   }
   // Known live because we hold a ref above in "editor"
   nsresult rv = MOZ_KnownLive(htmlEditor)
-                    ->PasteNoFormattingAsAction(nsIClipboard::kGlobalClipboard,
-                                                aPrincipal);
-  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
-                       "HTMLEditor::PasteNoFormattingAsAction() failed");
+                    ->PasteNoFormattingAsAction(
+                        nsIClipboard::kGlobalClipboard,
+                        EditorBase::DispatchPasteEvent::Yes, aPrincipal);
+  NS_WARNING_ASSERTION(
+      NS_SUCCEEDED(rv),
+      "HTMLEditor::PasteNoFormattingAsAction(DispatchPasteEvent::Yes) failed");
   return rv;
 }
 
@@ -144,18 +150,14 @@ nsresult PasteNoFormattingCommand::GetCommandStateParams(
 
 StaticRefPtr<StyleUpdatingCommand> StyleUpdatingCommand::sInstance;
 
-nsresult StyleUpdatingCommand::GetCurrentState(nsAtom* aTagName,
-                                               HTMLEditor* aHTMLEditor,
+nsresult StyleUpdatingCommand::GetCurrentState(nsStaticAtom& aTagName,
+                                               HTMLEditor& aHTMLEditor,
                                                nsCommandParams& aParams) const {
-  if (NS_WARN_IF(!aTagName) || NS_WARN_IF(!aHTMLEditor)) {
-    return NS_ERROR_INVALID_ARG;
-  }
-
   bool firstOfSelectionHasProp = false;
   bool anyOfSelectionHasProp = false;
   bool allOfSelectionHasProp = false;
 
-  nsresult rv = aHTMLEditor->GetInlineProperty(
+  nsresult rv = aHTMLEditor.GetInlineProperty(
       aTagName, nullptr, u""_ns, &firstOfSelectionHasProp,
       &anyOfSelectionHasProp, &allOfSelectionHasProp);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
@@ -183,7 +185,7 @@ nsresult StyleUpdatingCommand::ToggleState(nsStaticAtom& aTagName,
     doTagRemoval = true;
   } else {
     // check current selection; set doTagRemoval if formatting should be removed
-    nsresult rv = GetCurrentState(&aTagName, &aHTMLEditor, *params);
+    nsresult rv = GetCurrentState(aTagName, aHTMLEditor, *params);
     if (NS_FAILED(rv)) {
       NS_WARNING("StyleUpdatingCommand::GetCurrentState() failed");
       return rv;
@@ -216,21 +218,18 @@ nsresult StyleUpdatingCommand::ToggleState(nsStaticAtom& aTagName,
 
 StaticRefPtr<ListCommand> ListCommand::sInstance;
 
-nsresult ListCommand::GetCurrentState(nsAtom* aTagName, HTMLEditor* aHTMLEditor,
+nsresult ListCommand::GetCurrentState(nsStaticAtom& aTagName,
+                                      HTMLEditor& aHTMLEditor,
                                       nsCommandParams& aParams) const {
-  if (NS_WARN_IF(!aTagName) || NS_WARN_IF(!aHTMLEditor)) {
-    return NS_ERROR_INVALID_ARG;
-  }
-
   bool bMixed;
   nsAutoString localName;
-  nsresult rv = GetListState(aHTMLEditor, &bMixed, localName);
+  nsresult rv = GetListState(&aHTMLEditor, &bMixed, localName);
   if (NS_FAILED(rv)) {
     NS_WARNING("GetListState() failed");
     return rv;
   }
 
-  bool inList = aTagName->Equals(localName);
+  bool inList = aTagName.Equals(localName);
   aParams.SetBool(STATE_ALL, !bMixed && inList);
   aParams.SetBool(STATE_MIXED, bMixed);
   aParams.SetBool(STATE_ENABLED, true);
@@ -241,7 +240,7 @@ nsresult ListCommand::ToggleState(nsStaticAtom& aTagName,
                                   HTMLEditor& aHTMLEditor,
                                   nsIPrincipal* aPrincipal) const {
   RefPtr<nsCommandParams> params = new nsCommandParams();
-  nsresult rv = GetCurrentState(&aTagName, &aHTMLEditor, *params);
+  nsresult rv = GetCurrentState(aTagName, aHTMLEditor, *params);
   if (NS_FAILED(rv)) {
     NS_WARNING("ListCommand::GetCurrentState() failed");
     return rv;
@@ -274,15 +273,11 @@ nsresult ListCommand::ToggleState(nsStaticAtom& aTagName,
 
 StaticRefPtr<ListItemCommand> ListItemCommand::sInstance;
 
-nsresult ListItemCommand::GetCurrentState(nsAtom* aTagName,
-                                          HTMLEditor* aHTMLEditor,
+nsresult ListItemCommand::GetCurrentState(nsStaticAtom& aTagName,
+                                          HTMLEditor& aHTMLEditor,
                                           nsCommandParams& aParams) const {
-  if (NS_WARN_IF(!aTagName) || NS_WARN_IF(!aHTMLEditor)) {
-    return NS_ERROR_INVALID_ARG;
-  }
-
   ErrorResult error;
-  ListItemElementSelectionState state(*aHTMLEditor, error);
+  ListItemElementSelectionState state(aHTMLEditor, error);
   if (error.Failed()) {
     NS_WARNING("ListItemElementSelectionState failed");
     return error.StealNSResult();
@@ -302,7 +297,7 @@ nsresult ListItemCommand::GetCurrentState(nsAtom* aTagName,
   } else if (state.IsDDElementSelected()) {
     selectedListItemTagName = nsGkAtoms::dd;
   }
-  aParams.SetBool(STATE_ALL, aTagName == selectedListItemTagName);
+  aParams.SetBool(STATE_ALL, &aTagName == selectedListItemTagName);
   aParams.SetBool(STATE_MIXED, false);
   return NS_OK;
 }
@@ -312,7 +307,7 @@ nsresult ListItemCommand::ToggleState(nsStaticAtom& aTagName,
                                       nsIPrincipal* aPrincipal) const {
   // Need to use aTagName????
   RefPtr<nsCommandParams> params = new nsCommandParams();
-  GetCurrentState(&aTagName, &aHTMLEditor, *params);
+  GetCurrentState(aTagName, aHTMLEditor, *params);
   ErrorResult error;
   bool inList = params->GetBool(STATE_ALL, error);
   if (NS_WARN_IF(error.Failed())) {
@@ -341,7 +336,7 @@ nsresult ListItemCommand::ToggleState(nsStaticAtom& aTagName,
   // XXX Note: This actually doesn't work for "LI",
   //    but we currently don't use this for non DL lists anyway.
   // Problem: won't this replace any current block paragraph style?
-  nsresult rv = aHTMLEditor.SetParagraphFormatAsAction(
+  nsresult rv = aHTMLEditor.SetParagraphStateAsAction(
       nsDependentAtomString(&aTagName), aPrincipal);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                        "HTMLEditor::SetParagraphFormatAsAction() failed");
@@ -360,8 +355,7 @@ bool RemoveListCommand::IsCommandEnabled(Command aCommand,
   if (!htmlEditor) {
     return false;
   }
-
-  if (!htmlEditor->IsSelectionEditable()) {
+  if (!htmlEditor->IsModifiable() || !htmlEditor->IsSelectionEditable()) {
     return false;
   }
 
@@ -406,7 +400,7 @@ bool IndentCommand::IsCommandEnabled(Command aCommand,
   if (!htmlEditor) {
     return false;
   }
-  return htmlEditor->IsSelectionEditable();
+  return htmlEditor->IsModifiable() && htmlEditor->IsSelectionEditable();
 }
 
 nsresult IndentCommand::DoCommand(Command aCommand, EditorBase& aEditorBase,
@@ -439,7 +433,7 @@ bool OutdentCommand::IsCommandEnabled(Command aCommand,
   if (!htmlEditor) {
     return false;
   }
-  return htmlEditor->IsSelectionEditable();
+  return htmlEditor->IsModifiable() && htmlEditor->IsSelectionEditable();
 }
 
 nsresult OutdentCommand::DoCommand(Command aCommand, EditorBase& aEditorBase,
@@ -472,7 +466,7 @@ bool MultiStateCommandBase::IsCommandEnabled(Command aCommand,
     return false;
   }
   // should be disabled sometimes, like if the current selection is an image
-  return htmlEditor->IsSelectionEditable();
+  return htmlEditor->IsModifiable() && htmlEditor->IsSelectionEditable();
 }
 
 nsresult MultiStateCommandBase::DoCommand(Command aCommand,
@@ -516,6 +510,50 @@ nsresult MultiStateCommandBase::GetCommandStateParams(
   return rv;
 }
 
+/*****************************************************************************
+ * mozilla::FormatBlockStateCommand
+ *****************************************************************************/
+
+StaticRefPtr<FormatBlockStateCommand> FormatBlockStateCommand::sInstance;
+
+nsresult FormatBlockStateCommand::GetCurrentState(
+    HTMLEditor* aHTMLEditor, nsCommandParams& aParams) const {
+  if (NS_WARN_IF(!aHTMLEditor)) {
+    return NS_ERROR_INVALID_ARG;
+  }
+
+  ErrorResult error;
+  ParagraphStateAtSelection state(
+      *aHTMLEditor,
+      ParagraphStateAtSelection::FormatBlockMode::HTMLFormatBlockCommand,
+      error);
+  if (error.Failed()) {
+    NS_WARNING("ParagraphStateAtSelection failed");
+    return error.StealNSResult();
+  }
+  aParams.SetBool(STATE_MIXED, state.IsMixed());
+  if (NS_WARN_IF(!state.GetFirstParagraphStateAtSelection())) {
+    aParams.SetCString(STATE_ATTRIBUTE, ""_ns);
+  } else {
+    nsCString paragraphState;  // Don't use `nsAutoCString` for avoiding copy.
+    state.GetFirstParagraphStateAtSelection()->ToUTF8String(paragraphState);
+    aParams.SetCString(STATE_ATTRIBUTE, paragraphState);
+  }
+  return NS_OK;
+}
+
+nsresult FormatBlockStateCommand::SetState(HTMLEditor* aHTMLEditor,
+                                           const nsAString& aNewState,
+                                           nsIPrincipal* aPrincipal) const {
+  if (NS_WARN_IF(!aHTMLEditor) || NS_WARN_IF(aNewState.IsEmpty())) {
+    return NS_ERROR_INVALID_ARG;
+  }
+  nsresult rv = aHTMLEditor->FormatBlockAsAction(aNewState, aPrincipal);
+  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
+                       "HTMLEditor::FormatBlockAsAction() failed");
+  return rv;
+}
+
 /*****************************************************************************
  * mozilla::ParagraphStateCommand
  *****************************************************************************/
@@ -529,7 +567,10 @@ nsresult ParagraphStateCommand::GetCurrentState(
   }
 
   ErrorResult error;
-  ParagraphStateAtSelection state(*aHTMLEditor, error);
+  ParagraphStateAtSelection state(
+      *aHTMLEditor,
+      ParagraphStateAtSelection::FormatBlockMode::XULParagraphStateCommand,
+      error);
   if (error.Failed()) {
     NS_WARNING("ParagraphStateAtSelection failed");
     return error.StealNSResult();
@@ -552,9 +593,9 @@ nsresult ParagraphStateCommand::SetState(HTMLEditor* aHTMLEditor,
   if (NS_WARN_IF(!aHTMLEditor)) {
     return NS_ERROR_INVALID_ARG;
   }
-  nsresult rv = aHTMLEditor->SetParagraphFormatAsAction(aNewState, aPrincipal);
+  nsresult rv = aHTMLEditor->SetParagraphStateAsAction(aNewState, aPrincipal);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
-                       "HTMLEditor::SetParagraphFormatAsAction() failed");
+                       "HTMLEditor::SetParagraphStateAsAction() failed");
   return rv;
 }
 
@@ -621,7 +662,7 @@ nsresult FontSizeStateCommand::GetCurrentState(HTMLEditor* aHTMLEditor,
   nsAutoString outStateString;
   bool firstHas, anyHas, allHas;
   nsresult rv = aHTMLEditor->GetInlinePropertyWithAttrValue(
-      nsGkAtoms::font, nsGkAtoms::size, u""_ns, &firstHas, &anyHas, &allHas,
+      *nsGkAtoms::font, nsGkAtoms::size, u""_ns, &firstHas, &anyHas, &allHas,
       outStateString);
   if (NS_FAILED(rv)) {
     NS_WARNING(
@@ -884,19 +925,16 @@ nsresult AlignCommand::SetState(HTMLEditor* aHTMLEditor,
 StaticRefPtr<AbsolutePositioningCommand> AbsolutePositioningCommand::sInstance;
 
 nsresult AbsolutePositioningCommand::GetCurrentState(
-    nsAtom* aTagName, HTMLEditor* aHTMLEditor, nsCommandParams& aParams) const {
-  if (NS_WARN_IF(!aHTMLEditor)) {
-    return NS_ERROR_INVALID_ARG;
-  }
-
-  if (!aHTMLEditor->IsAbsolutePositionEditorEnabled()) {
+    nsStaticAtom& aTagName, HTMLEditor& aHTMLEditor,
+    nsCommandParams& aParams) const {
+  if (!aHTMLEditor.IsAbsolutePositionEditorEnabled()) {
     aParams.SetBool(STATE_MIXED, false);
     aParams.SetCString(STATE_ATTRIBUTE, ""_ns);
     return NS_OK;
   }
 
   RefPtr<Element> container =
-      aHTMLEditor->GetAbsolutelyPositionedSelectionContainer();
+      aHTMLEditor.GetAbsolutelyPositionedSelectionContainer();
   aParams.SetBool(STATE_MIXED, false);
   aParams.SetCString(STATE_ATTRIBUTE, container ? "absolute"_ns : ""_ns);
   return NS_OK;
@@ -1008,7 +1046,7 @@ bool RemoveStylesCommand::IsCommandEnabled(Command aCommand,
     return false;
   }
   // test if we have any styles?
-  return htmlEditor->IsSelectionEditable();
+  return htmlEditor->IsModifiable() && htmlEditor->IsSelectionEditable();
 }
 
 nsresult RemoveStylesCommand::DoCommand(Command aCommand,
@@ -1046,7 +1084,7 @@ bool IncreaseFontSizeCommand::IsCommandEnabled(Command aCommand,
     return false;
   }
   // test if we are at max size?
-  return htmlEditor->IsSelectionEditable();
+  return htmlEditor->IsModifiable() && htmlEditor->IsSelectionEditable();
 }
 
 nsresult IncreaseFontSizeCommand::DoCommand(Command aCommand,
@@ -1082,7 +1120,7 @@ bool DecreaseFontSizeCommand::IsCommandEnabled(Command aCommand,
     return false;
   }
   // test if we are at min size?
-  return htmlEditor->IsSelectionEditable();
+  return htmlEditor->IsModifiable() && htmlEditor->IsSelectionEditable();
 }
 
 nsresult DecreaseFontSizeCommand::DoCommand(Command aCommand,
@@ -1117,7 +1155,7 @@ bool InsertHTMLCommand::IsCommandEnabled(Command aCommand,
   if (!htmlEditor) {
     return false;
   }
-  return htmlEditor->IsSelectionEditable();
+  return htmlEditor->IsModifiable() && htmlEditor->IsSelectionEditable();
 }
 
 nsresult InsertHTMLCommand::DoCommand(Command aCommand, EditorBase& aEditorBase,
@@ -1174,7 +1212,7 @@ bool InsertTagCommand::IsCommandEnabled(Command aCommand,
   if (!htmlEditor) {
     return false;
   }
-  return htmlEditor->IsSelectionEditable();
+  return htmlEditor->IsModifiable() && htmlEditor->IsSelectionEditable();
 }
 
 // corresponding STATE_ATTRIBUTE is: src (img) and href (a)
