@@ -5,16 +5,18 @@
 
 #include "InsertTextTransaction.h"
 
+#include "ErrorList.h"
 #include "mozilla/EditorBase.h"  // mEditorBase
 #include "mozilla/Logging.h"
 #include "mozilla/SelectionState.h"  // RangeUpdater
 #include "mozilla/ToString.h"
 #include "mozilla/dom/Selection.h"  // Selection local var
 #include "mozilla/dom/Text.h"       // mTextNode
-#include "nsAString.h"              // nsAString parameter
-#include "nsDebug.h"                // for NS_ASSERTION, etc.
-#include "nsError.h"                // for NS_OK, etc.
-#include "nsQueryObject.h"          // for do_QueryObject
+
+#include "nsAString.h"      // nsAString parameter
+#include "nsDebug.h"        // for NS_ASSERTION, etc.
+#include "nsError.h"        // for NS_OK, etc.
+#include "nsQueryObject.h"  // for do_QueryObject
 
 namespace mozilla {
 
@@ -33,7 +35,7 @@ already_AddRefed<InsertTextTransaction> InsertTextTransaction::Create(
 InsertTextTransaction::InsertTextTransaction(
     EditorBase& aEditorBase, const nsAString& aStringToInsert,
     const EditorDOMPointInText& aPointToInsert)
-    : mTextNode(aPointToInsert.ContainerAsText()),
+    : mTextNode(aPointToInsert.ContainerAs<Text>()),
       mOffset(aPointToInsert.Offset()),
       mStringToInsert(aStringToInsert),
       mEditorBase(&aEditorBase) {}
@@ -77,24 +79,8 @@ NS_IMETHODIMP InsertTextTransaction::DoTransaction() {
     return error.StealNSResult();
   }
 
-  // Only set selection to insertion point if editor gives permission
-  if (editorBase->AllowsTransactionsToChangeSelection()) {
-    RefPtr<Selection> selection = editorBase->GetSelection();
-    if (NS_WARN_IF(!selection)) {
-      return NS_ERROR_FAILURE;
-    }
-    DebugOnly<nsresult> rvIgnored = selection->CollapseInLimiter(
-        textNode, mOffset + mStringToInsert.Length());
-    NS_ASSERTION(NS_SUCCEEDED(rvIgnored),
-                 "Selection::CollapseInLimiter() failed, but ignored");
-  } else {
-    // Do nothing - DOM Range gravity will adjust selection
-  }
-  // XXX Other transactions do not do this but its callers do.
-  //     Why do this transaction do this by itself?
   editorBase->RangeUpdaterRef().SelAdjInsertText(textNode, mOffset,
                                                  mStringToInsert.Length());
-
   return NS_OK;
 }
 
@@ -118,7 +104,22 @@ NS_IMETHODIMP InsertTextTransaction::RedoTransaction() {
   MOZ_LOG(GetLogModule(), LogLevel::Info,
           ("%p InsertTextTransaction::%s this=%s", this, __FUNCTION__,
            ToString(*this).c_str()));
-  return DoTransaction();
+  nsresult rv = DoTransaction();
+  if (NS_FAILED(rv)) {
+    NS_WARNING("InsertTextTransaction::DoTransaction() failed");
+    return rv;
+  }
+  if (RefPtr<EditorBase> editorBase = mEditorBase) {
+    nsresult rv = editorBase->CollapseSelectionTo(
+        SuggestPointToPutCaret<EditorRawDOMPoint>());
+    if (NS_WARN_IF(rv == NS_ERROR_EDITOR_DESTROYED)) {
+      return NS_ERROR_EDITOR_DESTROYED;
+    }
+    NS_WARNING_ASSERTION(
+        NS_SUCCEEDED(rv),
+        "EditorBase::CollapseSelectionTo() failed, but ignored");
+  }
+  return NS_OK;
 }
 
 NS_IMETHODIMP InsertTextTransaction::Merge(nsITransaction* aOtherTransaction,
@@ -156,9 +157,7 @@ NS_IMETHODIMP InsertTextTransaction::Merge(nsITransaction* aOtherTransaction,
     return NS_OK;
   }
 
-  nsAutoString otherData;
-  otherInsertTextTransaction->GetData(otherData);
-  mStringToInsert += otherData;
+  mStringToInsert += otherInsertTextTransaction->GetData();
   *aDidMerge = true;
   MOZ_LOG(GetLogModule(), LogLevel::Debug,
           ("%p InsertTextTransaction::%s(aOtherTransaction=%p) returned true",
@@ -168,12 +167,8 @@ NS_IMETHODIMP InsertTextTransaction::Merge(nsITransaction* aOtherTransaction,
 
 /* ============ private methods ================== */
 
-void InsertTextTransaction::GetData(nsString& aResult) {
-  aResult = mStringToInsert;
-}
-
 bool InsertTextTransaction::IsSequentialInsert(
-    InsertTextTransaction& aOtherTransaction) {
+    InsertTextTransaction& aOtherTransaction) const {
   return aOtherTransaction.mTextNode == mTextNode &&
          aOtherTransaction.mOffset == mOffset + mStringToInsert.Length();
 }
