@@ -7,6 +7,10 @@
 #define mozilla_TextEditor_h
 
 #include "mozilla/EditorBase.h"
+#include "mozilla/EditorForwards.h"
+#include "mozilla/TextControlState.h"
+#include "mozilla/UniquePtr.h"
+
 #include "nsCOMPtr.h"
 #include "nsCycleCollectionParticipant.h"
 #include "nsINamed.h"
@@ -22,12 +26,7 @@ class nsISelectionController;
 class nsITransferable;
 
 namespace mozilla {
-class DeleteNodeTransaction;
-class InsertNodeTransaction;
-enum class EditSubAction : int32_t;
-
 namespace dom {
-class DragEvent;
 class Selection;
 }  // namespace dom
 
@@ -35,7 +34,9 @@ class Selection;
  * The text editor implementation.
  * Use to edit text document represented as a DOM tree.
  */
-class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
+class TextEditor final : public EditorBase,
+                         public nsITimerCallback,
+                         public nsINamed {
  public:
   /****************************************************************************
    * NOTE: DO NOT MAKE YOUR NEW METHODS PUBLIC IF they are called by other
@@ -52,78 +53,77 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
 
   TextEditor();
 
+  /**
+   * Note that TextEditor::Init() shouldn't cause running script synchronously.
+   * So, `MOZ_CAN_RUN_SCRIPT_BOUNDARY` is safe here.
+   *
+   * @param aDocument   The document which aAnonymousDivElement belongs to.
+   * @param aAnonymousDivElement
+   *                    The root editable element for this editor.
+   * @param aSelectionController
+   *                    The selection controller for independent selections
+   *                    in the `<input>` or `<textarea>` element.
+   * @param aFlags      Some of nsIEditor::eEditor*Mask flags.
+   * @param aPasswordMaskData
+   *                    Set to an instance only when aFlags includes
+   *                    `nsIEditor::eEditorPasswordMask`.  Otherwise, must be
+   *                    `nullptr`.
+   */
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY nsresult
+  Init(Document& aDocument, Element& aAnonymousDivElement,
+       nsISelectionController& aSelectionController, uint32_t aFlags,
+       UniquePtr<PasswordMaskData>&& aPasswordMaskData);
+
+  /**
+   * PostCreate() should be called after Init, and is the time that the editor
+   * tells its documentStateObservers that the document has been created.
+   * Note that TextEditor::PostCreate() shouldn't cause running script
+   * synchronously. So, `MOZ_CAN_RUN_SCRIPT_BOUNDARY` is safe here.
+   */
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY nsresult PostCreate();
+
+  /**
+   * This method re-initializes the selection and caret state that are for
+   * current editor state. When editor session is destroyed, it always reset
+   * selection state even if this has no focus.  So if destroying editor,
+   * we have to call this method for focused editor to set selection state.
+   */
+  MOZ_CAN_RUN_SCRIPT void ReinitializeSelection(Element& aElement);
+
+  /**
+   * PreDestroy() is called before the editor goes away, and gives the editor a
+   * chance to tell its documentStateObservers that the document is going away.
+   * Note that TextEditor::PreDestroy() shouldn't cause running script
+   * synchronously. So, `MOZ_CAN_RUN_SCRIPT_BOUNDARY` is safe here.
+   */
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT_BOUNDARY UniquePtr<PasswordMaskData>
+  PreDestroy();
+
+  static TextEditor* GetFrom(nsIEditor* aEditor) {
+    return aEditor ? aEditor->GetAsTextEditor() : nullptr;
+  }
+  static const TextEditor* GetFrom(const nsIEditor* aEditor) {
+    return aEditor ? aEditor->GetAsTextEditor() : nullptr;
+  }
+
+  /**
+   * Helper method for `AppendString()` and `AppendSubString()`.  This should
+   * be called only when `aText` is in a password field.  This method masks
+   * A part of or all of `aText` (`aStartOffsetInText` and later) should've
+   * been copied (appended) to `aString`.  `aStartOffsetInString` is where
+   * the password was appended into `aString`.
+   */
+  static void MaskString(nsString& aString, const dom::Text& aTextNode,
+                         uint32_t aStartOffsetInString,
+                         uint32_t aStartOffsetInText);
+
   NS_DECL_NSITIMERCALLBACK
   NS_DECL_NSINAMED
 
   // Overrides of nsIEditor
-  NS_IMETHOD GetDocumentIsEmpty(bool* aDocumentIsEmpty) override;
-
-  MOZ_CAN_RUN_SCRIPT NS_IMETHOD
-  SetDocumentCharacterSet(const nsACString& characterSet) override;
-
-  NS_IMETHOD GetTextLength(int32_t* aCount) override;
-
-  /**
-   * Do "undo" or "redo".
-   *
-   * @param aCount              How many count of transactions should be
-   *                            handled.
-   * @param aPrincipal          Set subject principal if it may be called by
-   *                            JS.  If set to nullptr, will be treated as
-   *                            called by system.
-   */
-  MOZ_CAN_RUN_SCRIPT nsresult UndoAsAction(uint32_t aCount,
-                                           nsIPrincipal* aPrincipal = nullptr);
-  MOZ_CAN_RUN_SCRIPT nsresult RedoAsAction(uint32_t aCount,
-                                           nsIPrincipal* aPrincipal = nullptr);
-
-  /**
-   * Do "cut".
-   *
-   * @param aPrincipal          If you know current context is subject
-   *                            principal or system principal, set it.
-   *                            When nullptr, this checks it automatically.
-   */
-  MOZ_CAN_RUN_SCRIPT nsresult CutAsAction(nsIPrincipal* aPrincipal = nullptr);
-
-  /**
-   * IsCutCommandEnabled() returns whether cut command can be enabled or
-   * disabled.  This always returns true if we're in non-chrome HTML/XHTML
-   * document.  Otherwise, same as the result of `IsCopyToClipboardAllowed()`.
-   */
-  bool IsCutCommandEnabled() const;
-
-  NS_IMETHOD Copy() override;
-
-  /**
-   * IsCopyCommandEnabled() returns copy command can be enabled or disabled.
-   * This always returns true if we're in non-chrome HTML/XHTML document.
-   * Otherwise, same as the result of `IsCopyToClipboardAllowed()`.
-   */
-  bool IsCopyCommandEnabled() const;
-
-  /**
-   * IsCopyToClipboardAllowed() returns true if the selected content can
-   * be copied into the clipboard.  This returns true when:
-   * - `Selection` is not collapsed and we're not a password editor.
-   * - `Selection` is not collapsed and we're a password editor but selection
-   *   range is in unmasked range.
-   */
-  bool IsCopyToClipboardAllowed() const {
-    AutoEditActionDataSetter editActionData(*this, EditAction::eNotEditing);
-    if (NS_WARN_IF(!editActionData.CanHandle())) {
-      return false;
-    }
-    return IsCopyToClipboardAllowedInternal();
-  }
-
-  /**
-   * CanDeleteSelection() returns true if `Selection` is not collapsed and
-   * it's allowed to be removed.
-   */
-  bool CanDeleteSelection() const;
-
-  virtual bool CanPaste(int32_t aClipboardType) const;
+  MOZ_CAN_RUN_SCRIPT NS_IMETHOD EndOfDocument() final;
+  MOZ_CAN_RUN_SCRIPT NS_IMETHOD InsertLineBreak() final;
+  NS_IMETHOD GetTextLength(uint32_t* aCount) final;
 
   // Shouldn't be used internally, but we need these using declarations for
   // avoiding warnings of clang.
@@ -131,78 +131,22 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
   using EditorBase::CanCut;
   using EditorBase::CanPaste;
 
-  /**
-   * Paste aTransferable at Selection.
-   *
-   * @param aTransferable       Must not be nullptr.
-   * @param aPrincipal          Set subject principal if it may be called by
-   *                            JS.  If set to nullptr, will be treated as
-   *                            called by system.
-   */
-  MOZ_CAN_RUN_SCRIPT virtual nsresult PasteTransferableAsAction(
-      nsITransferable* aTransferable, nsIPrincipal* aPrincipal = nullptr);
-
-  NS_IMETHOD OutputToString(const nsAString& aFormatType, uint32_t aFlags,
-                            nsAString& aOutputString) override;
-
-  /** Can we paste |aTransferable| or, if |aTransferable| is null, will a call
-   * to pasteTransferable later possibly succeed if given an instance of
-   * nsITransferable then? True if the doc is modifiable, and, if
-   * |aTransfeable| is non-null, we have pasteable data in |aTransfeable|.
-   */
-  virtual bool CanPasteTransferable(nsITransferable* aTransferable);
-
   // Overrides of EditorBase
-  MOZ_CAN_RUN_SCRIPT virtual nsresult Init(Document& aDoc, Element* aRoot,
-                                           nsISelectionController* aSelCon,
-                                           uint32_t aFlags,
-                                           const nsAString& aValue) override;
+  bool IsEmpty() const final;
 
-  /**
-   * IsEmpty() checks whether the editor is empty.  If editor has only padding
-   * <br> element for empty editor, returns true.  If editor's root element has
-   * non-empty text nodes or other nodes like <br>, returns false.
-   */
-  virtual bool IsEmpty() const;
+  bool CanPaste(int32_t aClipboardType) const final;
 
-  MOZ_CAN_RUN_SCRIPT virtual nsresult HandleKeyPressEvent(
-      WidgetKeyboardEvent* aKeyboardEvent) override;
+  bool CanPasteTransferable(nsITransferable* aTransferable) final;
 
-  virtual dom::EventTarget* GetDOMEventTarget() const override;
+  MOZ_CAN_RUN_SCRIPT nsresult
+  HandleKeyPressEvent(WidgetKeyboardEvent* aKeyboardEvent) final;
 
-  /**
-   * PasteAsAction() pastes clipboard content to Selection.  This method
-   * may dispatch ePaste event first.  If its defaultPrevent() is called,
-   * this does nothing but returns NS_OK.
-   *
-   * @param aClipboardType      nsIClipboard::kGlobalClipboard or
-   *                            nsIClipboard::kSelectionClipboard.
-   * @param aDispatchPasteEvent true if this should dispatch ePaste event
-   *                            before pasting.  Otherwise, false.
-   * @param aPrincipal          Set subject principal if it may be called by
-   *                            JS.  If set to nullptr, will be treated as
-   *                            called by system.
-   */
-  MOZ_CAN_RUN_SCRIPT nsresult PasteAsAction(int32_t aClipboardType,
-                                            bool aDispatchPasteEvent,
-                                            nsIPrincipal* aPrincipal = nullptr);
+  dom::EventTarget* GetDOMEventTarget() const final;
 
-  /**
-   * PasteAsQuotationAsAction() pastes content in clipboard as quotation.
-   * If the editor is TextEditor or in plaintext mode, will paste the content
-   * with appending ">" to start of each line.
-   *
-   * @param aClipboardType      nsIClipboard::kGlobalClipboard or
-   *                            nsIClipboard::kSelectionClipboard.
-   * @param aDispatchPasteEvent true if this should dispatch ePaste event
-   *                            before pasting.  Otherwise, false.
-   * @param aPrincipal          Set subject principal if it may be called by
-   *                            JS.  If set to nullptr, will be treated as
-   *                            called by system.
-   */
-  MOZ_CAN_RUN_SCRIPT virtual nsresult PasteAsQuotationAsAction(
-      int32_t aClipboardType, bool aDispatchPasteEvent,
-      nsIPrincipal* aPrincipal = nullptr);
+  MOZ_CAN_RUN_SCRIPT nsresult
+  OnFocus(const nsINode& aOriginalEventTargetNode) final;
+
+  nsresult OnBlur(const dom::EventTarget* aEventTarget) final;
 
   /**
    * The maximum number of characters allowed.
@@ -211,73 +155,31 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
   int32_t MaxTextLength() const { return mMaxTextLength; }
   void SetMaxTextLength(int32_t aLength) { mMaxTextLength = aLength; }
 
+  /**
+   * This updates the wrap width used for initializing a document encoder within
+   * a call of EditorBase::GetAndInitDocEncoder().
+   */
+  void SetWrapColumn(int32_t aWrapColumn) { mWrapColumn = aWrapColumn; }
+
   /**
    * Replace existed string with a string.
    * This is fast path to replace all string when using single line control.
    *
    * @param aString             The string to be set
+   * @param aAllowBeforeInputEventCancelable
+   *                            Whether `beforeinput` event which will be
+   *                            dispatched for this can be cancelable or not.
    * @param aPrincipal          Set subject principal if it may be called by
    *                            JS.  If set to nullptr, will be treated as
    *                            called by system.
    */
-  MOZ_CAN_RUN_SCRIPT nsresult
-  SetTextAsAction(const nsAString& aString, nsIPrincipal* aPrincipal = nullptr);
-
-  /**
-   * Replace text in aReplaceRange or all text in this editor with aString and
-   * treat the change as inserting the string.
-   *
-   * @param aString             The string to set.
-   * @param aReplaceRange       The range to be replaced.
-   *                            If nullptr, all contents will be replaced.
-   * @param aPrincipal          Set subject principal if it may be called by
-   *                            JS.  If set to nullptr, will be treated as
-   *                            called by system.
-   */
-  MOZ_CAN_RUN_SCRIPT nsresult
-  ReplaceTextAsAction(const nsAString& aString, nsRange* aReplaceRange,
-                      nsIPrincipal* aPrincipal = nullptr);
-
-  /**
-   * InsertLineBreakAsAction() is called when user inputs a line break with
-   * Enter or something.
-   *
-   * @param aPrincipal          Set subject principal if it may be called by
-   *                            JS.  If set to nullptr, will be treated as
-   *                            called by system.
-   */
-  MOZ_CAN_RUN_SCRIPT virtual nsresult InsertLineBreakAsAction(
+  MOZ_CAN_RUN_SCRIPT nsresult SetTextAsAction(
+      const nsAString& aString,
+      AllowBeforeInputEventCancelable aAllowBeforeInputEventCancelable,
       nsIPrincipal* aPrincipal = nullptr);
 
-  /**
-   * OnCompositionStart() is called when editor receives eCompositionStart
-   * event which should be handled in this editor.
-   */
-  nsresult OnCompositionStart(WidgetCompositionEvent& aCompositionStartEvent);
-
-  /**
-   * OnCompositionChange() is called when editor receives an eCompositioChange
-   * event which should be handled in this editor.
-   *
-   * @param aCompositionChangeEvent     eCompositionChange event which should
-   *                                    be handled in this editor.
-   */
   MOZ_CAN_RUN_SCRIPT nsresult
-  OnCompositionChange(WidgetCompositionEvent& aCompositionChangeEvent);
-
-  /**
-   * OnCompositionEnd() is called when editor receives an eCompositionChange
-   * event and it's followed by eCompositionEnd event and after
-   * OnCompositionChange() is called.
-   */
-  MOZ_CAN_RUN_SCRIPT void OnCompositionEnd(
-      WidgetCompositionEvent& aCompositionEndEvent);
-
-  /**
-   * OnDrop() is called from EditorEventListener::Drop that is handler of drop
-   * event.
-   */
-  MOZ_CAN_RUN_SCRIPT nsresult OnDrop(dom::DragEvent* aDropEvent);
+  InsertLineBreakAsAction(nsIPrincipal* aPrincipal = nullptr) final;
 
   /**
    * ComputeTextValue() computes plaintext value of this editor.  This may be
@@ -292,8 +194,8 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
     if (NS_WARN_IF(!editActionData.CanHandle())) {
       return NS_ERROR_NOT_INITIALIZED;
     }
-    nsresult rv = ComputeValueInternal(NS_LITERAL_STRING("text/plain"),
-                                       aDocumentEncoderFlags, aOutputString);
+    nsresult rv = ComputeValueInternal(u"text/plain"_ns, aDocumentEncoderFlags,
+                                       aOutputString);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return EditorBase::ToGenericNSResult(rv);
     }
@@ -305,21 +207,21 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
    * editor.  They return whether there is unmasked range or not and range
    * start and length.
    */
-  bool IsAllMasked() const {
+  MOZ_ALWAYS_INLINE bool IsAllMasked() const {
     MOZ_ASSERT(IsPasswordEditor());
-    return mUnmaskedStart == UINT32_MAX && mUnmaskedLength == 0;
+    return !mPasswordMaskData || mPasswordMaskData->IsAllMasked();
   }
-  uint32_t UnmaskedStart() const {
+  MOZ_ALWAYS_INLINE uint32_t UnmaskedStart() const {
     MOZ_ASSERT(IsPasswordEditor());
-    return mUnmaskedStart;
+    return mPasswordMaskData ? mPasswordMaskData->mUnmaskedStart : UINT32_MAX;
   }
-  uint32_t UnmaskedLength() const {
+  MOZ_ALWAYS_INLINE uint32_t UnmaskedLength() const {
     MOZ_ASSERT(IsPasswordEditor());
-    return mUnmaskedLength;
+    return mPasswordMaskData ? mPasswordMaskData->mUnmaskedLength : 0;
   }
-  uint32_t UnmaskedEnd() const {
+  MOZ_ALWAYS_INLINE uint32_t UnmaskedEnd() const {
     MOZ_ASSERT(IsPasswordEditor());
-    return mUnmaskedStart + mUnmaskedLength;
+    return mPasswordMaskData ? mPasswordMaskData->UnmaskedEnd() : UINT32_MAX;
   }
 
   /**
@@ -329,7 +231,7 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
    */
   bool IsMaskingPassword() const {
     MOZ_ASSERT(IsPasswordEditor());
-    return mIsMaskingPassword;
+    return mPasswordMaskData && mPasswordMaskData->mIsMaskingPassword;
   }
 
   /**
@@ -338,6 +240,37 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
    */
   static char16_t PasswordMask();
 
+  /**
+   * If you want to prevent to echo password temporarily, use the following
+   * methods.
+   */
+  bool EchoingPasswordPrevented() const {
+    return mPasswordMaskData && mPasswordMaskData->mEchoingPasswordPrevented;
+  }
+  void PreventToEchoPassword() {
+    if (mPasswordMaskData) {
+      mPasswordMaskData->mEchoingPasswordPrevented = true;
+    }
+  }
+  void AllowToEchoPassword() {
+    if (mPasswordMaskData) {
+      mPasswordMaskData->mEchoingPasswordPrevented = false;
+    }
+  }
+
+  dom::Text* GetTextNode() {
+    MOZ_DIAGNOSTIC_ASSERT(GetRoot());
+    MOZ_DIAGNOSTIC_ASSERT(GetRoot()->GetFirstChild());
+    MOZ_DIAGNOSTIC_ASSERT(GetRoot()->GetFirstChild()->IsText());
+    if (MOZ_UNLIKELY(!GetRoot() || !GetRoot()->GetFirstChild())) {
+      return nullptr;
+    }
+    return GetRoot()->GetFirstChild()->GetAsText();
+  }
+  const dom::Text* GetTextNode() const {
+    return const_cast<TextEditor*>(this)->GetTextNode();
+  }
+
  protected:  // May be called by friends.
   /****************************************************************************
    * Some friend classes are allowed to call the following protected methods.
@@ -347,21 +280,30 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
    ****************************************************************************/
 
   // Overrides of EditorBase
-  MOZ_CAN_RUN_SCRIPT virtual nsresult RemoveAttributeOrEquivalent(
-      Element* aElement, nsAtom* aAttribute,
-      bool aSuppressTransaction) override;
-  MOZ_CAN_RUN_SCRIPT virtual nsresult SetAttributeOrEquivalent(
+  MOZ_CAN_RUN_SCRIPT nsresult RemoveAttributeOrEquivalent(
+      Element* aElement, nsAtom* aAttribute, bool aSuppressTransaction) final;
+  MOZ_CAN_RUN_SCRIPT nsresult SetAttributeOrEquivalent(
       Element* aElement, nsAtom* aAttribute, const nsAString& aValue,
-      bool aSuppressTransaction) override;
+      bool aSuppressTransaction) final;
   using EditorBase::RemoveAttributeOrEquivalent;
   using EditorBase::SetAttributeOrEquivalent;
 
   /**
-   * DeleteSelectionByDragAsAction() removes selection and dispatch "input"
-   * event whose inputType is "deleteByDrag".
+   * FindBetterInsertionPoint() tries to look for better insertion point which
+   * is typically the nearest text node and offset in it.
+   *
+   * @param aPoint      Insertion point which the callers found.
+   * @return            Better insertion point if there is.  If not returns
+   *                    same point as aPoint.
    */
-  [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult
-  DeleteSelectionByDragAsAction(bool aDispatchInputEvent);
+  template <typename EditorDOMPointType>
+  EditorDOMPointType FindBetterInsertionPoint(
+      const EditorDOMPointType& aPoint) const;
+
+  /**
+   * InsertLineBreakAsSubAction() inserts a line break.
+   */
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult InsertLineBreakAsSubAction();
 
   /**
    * Replace existed string with aString.  Caller must guarantee that there
@@ -372,14 +314,6 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
   [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult
   SetTextAsSubAction(const nsAString& aString);
 
-  /**
-   * ReplaceSelectionAsSubAction() replaces selection with aString.
-   *
-   * @param aString    The string to replace.
-   */
-  MOZ_CAN_RUN_SCRIPT nsresult
-  ReplaceSelectionAsSubAction(const nsAString& aString);
-
   /**
    * MaybeDoAutoPasswordMasking() may mask password if we're doing auto-masking.
    */
@@ -438,6 +372,9 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
    * After this is called, TextEditor starts masking password automatically.
    */
   MOZ_CAN_RUN_SCRIPT_BOUNDARY nsresult MaskAllCharacters() {
+    if (!mPasswordMaskData) {
+      return NS_OK;  // Already we don't have masked range data.
+    }
     return SetUnmaskRangeInternal(UINT32_MAX, 0, 0, false, true);
   }
 
@@ -489,14 +426,14 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
    * @return                    If aInsertionString is truncated, it returns "as
    *                            handled", else "as ignored."
    */
-  EditActionResult MaybeTruncateInsertionStringForMaxLength(
+  Result<EditActionResult, nsresult> MaybeTruncateInsertionStringForMaxLength(
       nsAString& aInsertionString);
 
   /**
    * InsertLineFeedCharacterAtSelection() inserts a linefeed character at
    * selection.
    */
-  [[nodiscard]] MOZ_CAN_RUN_SCRIPT EditActionResult
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT Result<EditActionResult, nsresult>
   InsertLineFeedCharacterAtSelection();
 
   /**
@@ -510,7 +447,7 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
    *   nsIEditor::eNewlinesReplaceWithCommas (4, Thunderbird default):
    *     replace newlines with commas.
    *   nsIEditor::eNewlinesStripSurroundingWhitespace (5):
-   *     collapse newlines and surrounding whitespace characters and
+   *     collapse newlines and surrounding white-space characters and
    *     remove them from the string.
    *   nsIEditor::eNewlinesPasteIntact (0):
    *     only remove the leading and trailing newlines.
@@ -521,15 +458,15 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
    */
   void HandleNewLinesInStringForSingleLineEditor(nsString& aString) const;
 
-  /**
-   * HandleInsertText() handles inserting text at selection.
-   *
-   * @param aEditSubAction      Must be EditSubAction::eInsertText or
-   *                            EditSubAction::eInsertTextComingFromIME.
-   * @param aInsertionString    String to be inserted at selection.
-   */
-  [[nodiscard]] MOZ_CAN_RUN_SCRIPT virtual EditActionResult HandleInsertText(
-      EditSubAction aEditSubAction, const nsAString& aInsertionString);
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT Result<EditActionResult, nsresult>
+  HandleInsertText(EditSubAction aEditSubAction,
+                   const nsAString& aInsertionString,
+                   SelectionHandling aSelectionHandling) final;
+
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult InsertDroppedDataTransferAsAction(
+      AutoEditActionDataSetter& aEditActionData,
+      dom::DataTransfer& aDataTransfer, const EditorDOMPoint& aDroppedAt,
+      nsIPrincipal* aSourcePrincipal) final;
 
   /**
    * HandleDeleteSelectionInternal() is a helper method of
@@ -539,7 +476,7 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
    *       needs to check if the editor is still available even if this returns
    *       NS_OK.
    */
-  [[nodiscard]] MOZ_CAN_RUN_SCRIPT EditActionResult
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT Result<EditActionResult, nsresult>
   HandleDeleteSelectionInternal(nsIEditor::EDirection aDirectionAndAmount,
                                 nsIEditor::EStripWrappers aStripWrappers);
 
@@ -549,18 +486,18 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
    * @param aDirectionAndAmount Direction of the deletion.
    * @param aStripWrappers      Must be nsIEditor::eNoStrip.
    */
-  [[nodiscard]] MOZ_CAN_RUN_SCRIPT EditActionResult
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT Result<EditActionResult, nsresult>
   HandleDeleteSelection(nsIEditor::EDirection aDirectionAndAmount,
-                        nsIEditor::EStripWrappers aStripWrappers) override;
+                        nsIEditor::EStripWrappers aStripWrappers) final;
 
   /**
-   * ComputeValueFromTextNodeAndPaddingBRElement() tries to compute "value" of
-   * this editor content only with text node and padding `<br>` element.
+   * ComputeValueFromTextNodeAndBRElement() tries to compute "value" of
+   * this editor content only with text nodes and `<br>` elements.
    * If this succeeds to compute the value, it's returned with aValue and
    * the result is marked as "handled".  Otherwise, the caller needs to
    * compute it with another way.
    */
-  EditActionResult ComputeValueFromTextNodeAndPaddingBRElement(
+  Result<EditActionResult, nsresult> ComputeValueFromTextNodeAndBRElement(
       nsAString& aValue) const;
 
   /**
@@ -568,16 +505,9 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
    * and `<textarea>.value` to aValue without transaction.  This must be
    * called only when it's not `HTMLEditor` and undo/redo is disabled.
    */
-  [[nodiscard]] MOZ_CAN_RUN_SCRIPT EditActionResult
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT Result<EditActionResult, nsresult>
   SetTextWithoutTransaction(const nsAString& aValue);
 
-  /**
-   * EnsurePaddingBRElementInMultilineEditor() creates a padding `<br>` element
-   * at end of multiline text editor.
-   */
-  [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult
-  EnsurePaddingBRElementInMultilineEditor();
-
   /**
    * EnsureCaretNotAtEndOfTextNode() collapses selection at the padding `<br>`
    * element (i.e., container becomes the anonymous `<div>` element) if
@@ -586,19 +516,11 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
   [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult EnsureCaretNotAtEndOfTextNode();
 
  protected:  // Called by helper classes.
-  MOZ_CAN_RUN_SCRIPT virtual void OnStartToHandleTopLevelEditSubAction(
+  MOZ_CAN_RUN_SCRIPT void OnStartToHandleTopLevelEditSubAction(
       EditSubAction aTopLevelEditSubAction,
       nsIEditor::EDirection aDirectionOfTopLevelEditSubAction,
-      ErrorResult& aRv) override;
-  MOZ_CAN_RUN_SCRIPT virtual nsresult OnEndHandlingTopLevelEditSubAction()
-      override;
-
-  /**
-   * EnsurePaddingBRElementForEmptyEditor() creates padding <br> element for
-   * empty editor or changes padding <br> element for empty last line to for
-   * empty editor when we're empty.
-   */
-  MOZ_CAN_RUN_SCRIPT nsresult EnsurePaddingBRElementForEmptyEditor();
+      ErrorResult& aRv) final;
+  MOZ_CAN_RUN_SCRIPT nsresult OnEndHandlingTopLevelEditSubAction() final;
 
   /**
    * HandleInlineSpellCheckAfterEdit() does spell-check after handling top level
@@ -618,13 +540,6 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
  protected:  // Shouldn't be used by friend classes
   virtual ~TextEditor();
 
-  /**
-   * InitEditorContentAndSelection() may insert a padding `<br>` element for
-   * if it's required in the anonymous `<div>` element and collapse selection
-   * at the end if there is no selection ranges.
-   */
-  [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult InitEditorContentAndSelection();
-
   /**
    * CanEchoPasswordNow() returns true if currently we can echo password.
    * If it's direct user input such as pasting or dropping text, this
@@ -632,128 +547,41 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
    */
   bool CanEchoPasswordNow() const;
 
+  /**
+   * InitEditorContentAndSelection() may insert a padding `<br>` element for
+   * if it's required in the anonymous `<div>` element or `<body>` element and
+   * collapse selection at the end if there is no selection ranges.
+   */
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult InitEditorContentAndSelection();
+
+  /**
+   * Collapse `Selection` to end of the text node in the anonymous <div>
+   * element.
+   */
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult CollapseSelectionToEndOfTextNode();
+
   /**
    * Make the given selection span the entire document.
    */
-  MOZ_CAN_RUN_SCRIPT virtual nsresult SelectEntireDocument() override;
+  MOZ_CAN_RUN_SCRIPT nsresult SelectEntireDocument() final;
 
-  /**
-   * OnInputText() is called when user inputs text with keyboard or something.
-   *
-   * @param aStringToInsert     The string to insert.
-   */
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult HandlePaste(
+      AutoEditActionDataSetter& aEditActionData, int32_t aClipboardType) final;
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult HandlePasteAsQuotation(
+      AutoEditActionDataSetter& aEditActionData, int32_t aClipboardType) final;
   [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult
-  OnInputText(const nsAString& aStringToInsert);
+  HandlePasteTransferable(AutoEditActionDataSetter& aEditActionData,
+                          nsITransferable& aTransferable) final;
 
-  /**
-   * PrepareInsertContent() is a helper method of InsertTextAt(),
-   * HTMLEditor::DoInsertHTMLWithContext().  They insert content coming from
-   * clipboard or drag and drop.  Before that, they may need to remove selected
-   * contents and adjust selection.  This does them instead.
-   *
-   * @param aPointToInsert      Point to insert.  Must be set.  Callers
-   *                            shouldn't use this instance after calling this
-   *                            method because this method may cause changing
-   *                            the DOM tree and Selection.
-   * @param aDoDeleteSelection  true if selected content should be removed.
-   */
-  MOZ_CAN_RUN_SCRIPT nsresult PrepareToInsertContent(
-      const EditorDOMPoint& aPointToInsert, bool aDoDeleteSelection);
-
-  /**
-   * InsertTextAt() inserts aStringToInsert at aPointToInsert.
-   *
-   * @param aStringToInsert     The string which you want to insert.
-   * @param aPointToInsert      The insertion point.
-   * @param aDoDeleteSelection  true if you want this to delete selected
-   *                            content.  Otherwise, false.
-   */
-  MOZ_CAN_RUN_SCRIPT nsresult InsertTextAt(const nsAString& aStringToInsert,
-                                           const EditorDOMPoint& aPointToInsert,
-                                           bool aDoDeleteSelection);
-
-  /**
-   * InsertWithQuotationsAsSubAction() inserts aQuotedText with appending ">"
-   * to start of every line.
-   *
-   * @param aQuotedText         String to insert.  This will be quoted by ">"
-   *                            automatically.
-   */
-  [[nodiscard]] MOZ_CAN_RUN_SCRIPT virtual nsresult
-  InsertWithQuotationsAsSubAction(const nsAString& aQuotedText);
-
-  /**
-   * Return true if the data is safe to insert as the source and destination
-   * principals match, or we are in a editor context where this doesn't matter.
-   * Otherwise, the data must be sanitized first.
-   */
-  bool IsSafeToInsertData(Document* aSourceDoc);
-
-  /**
-   * GetAndInitDocEncoder() returns a document encoder instance for aFormatType
-   * after initializing it.  The result may be cached for saving recreation
-   * cost.
-   *
-   * @param aFormatType             MIME type like "text/plain".
-   * @param aDocumentEncoderFlags   Flags of nsIDocumentEncoder.
-   * @param aCharset                Encoding of the document.
-   */
-  already_AddRefed<nsIDocumentEncoder> GetAndInitDocEncoder(
-      const nsAString& aFormatType, uint32_t aDocumentEncoderFlags,
-      const nsACString& aCharset) const;
-
-  /**
-   * ComputeValueInternal() computes string value of this editor for given
-   * format.  This may be too expensive if it's in hot path.
-   *
-   * @param aFormatType             MIME type like "text/plain".
-   * @param aDocumentEncoderFlags   Flags of nsIDocumentEncoder.
-   * @param aCharset                Encoding of the document.
-   */
-  nsresult ComputeValueInternal(const nsAString& aFormatType,
-                                uint32_t aDocumentEncoderFlags,
-                                nsAString& aOutputString) const;
-
-  /**
-   * Factored methods for handling insertion of data from transferables
-   * (drag&drop or clipboard).
-   */
-  virtual nsresult PrepareTransferable(nsITransferable** transferable);
+  [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult
+  InsertWithQuotationsAsSubAction(const nsAString& aQuotedText) final;
 
   [[nodiscard]] MOZ_CAN_RUN_SCRIPT nsresult
   InsertTextFromTransferable(nsITransferable* transferable);
 
-  /**
-   * Shared outputstring; returns whether selection is collapsed and resulting
-   * string.
-   */
-  nsresult SharedOutputString(uint32_t aFlags, bool* aIsCollapsed,
-                              nsAString& aResult) const;
+  bool IsCopyToClipboardAllowedInternal() const final;
 
-  /**
-   * See comment of IsCopyToClipboardAllowed() for the detail.
-   */
-  bool IsCopyToClipboardAllowedInternal() const;
-
-  bool FireClipboardEvent(EventMessage aEventMessage, int32_t aSelectionType,
-                          bool* aActionTaken = nullptr);
-
-  MOZ_CAN_RUN_SCRIPT bool UpdateMetaCharset(Document& aDocument,
-                                            const nsACString& aCharacterSet);
-
-  /**
-   * EnsureComposition() should be called by composition event handlers.  This
-   * tries to get the composition for the event and set it to mComposition.
-   * However, this may fail because the composition may be committed before
-   * the event comes to the editor.
-   *
-   * @return            true if there is a composition.  Otherwise, for example,
-   *                    a composition event handler in web contents moved focus
-   *                    for committing the composition, returns false.
-   */
-  bool EnsureComposition(WidgetCompositionEvent& aCompositionEvent);
-
-  virtual already_AddRefed<Element> GetInputEventTargetElement() const override;
+  already_AddRefed<Element> GetInputEventTargetElement() const final;
 
   /**
    * See SetUnmaskRange() and SetUnmaskRangeAndNotify() for the detail.
@@ -767,27 +595,16 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
                                                      bool aNotify,
                                                      bool aForceStartMasking);
 
+  MOZ_ALWAYS_INLINE bool HasAutoMaskingTimer() const {
+    return mPasswordMaskData && mPasswordMaskData->mTimer;
+  }
+
  protected:
-  mutable nsCOMPtr<nsIDocumentEncoder> mCachedDocumentEncoder;
+  UniquePtr<PasswordMaskData> mPasswordMaskData;
 
-  // Timer to mask unmasked characters automatically.  Used only when it's
-  // a password field.
-  nsCOMPtr<nsITimer> mMaskTimer;
-
-  mutable nsString mCachedDocumentEncoderType;
-
-  int32_t mMaxTextLength;
-
-  // Unmasked character range.  Used only when it's a password field.
-  // If mUnmaskedLength is 0, it means there is no unmasked characters.
-  uint32_t mUnmaskedStart;
-  uint32_t mUnmaskedLength;
-
-  // Set to true if all characters are masked or waiting notification from
-  // `mMaskTimer`.  Otherwise, i.e., part of or all of password is unmasked
-  // without setting `mMaskTimer`, set to false.
-  bool mIsMaskingPassword;
+  int32_t mMaxTextLength = -1;
 
+  friend class AutoRangeArray;  // FindBetterInsertionPoint
   friend class DeleteNodeTransaction;
   friend class EditorBase;
   friend class InsertNodeTransaction;
@@ -796,11 +613,21 @@ class TextEditor : public EditorBase, public nsITimerCallback, public nsINamed {
 }  // namespace mozilla
 
 mozilla::TextEditor* nsIEditor::AsTextEditor() {
+  MOZ_DIAGNOSTIC_ASSERT(IsTextEditor());
   return static_cast<mozilla::TextEditor*>(this);
 }
 
 const mozilla::TextEditor* nsIEditor::AsTextEditor() const {
+  MOZ_DIAGNOSTIC_ASSERT(IsTextEditor());
   return static_cast<const mozilla::TextEditor*>(this);
 }
 
+mozilla::TextEditor* nsIEditor::GetAsTextEditor() {
+  return AsEditorBase()->IsTextEditor() ? AsTextEditor() : nullptr;
+}
+
+const mozilla::TextEditor* nsIEditor::GetAsTextEditor() const {
+  return AsEditorBase()->IsTextEditor() ? AsTextEditor() : nullptr;
+}
+
 #endif  // #ifndef mozilla_TextEditor_h
