@@ -2,24 +2,26 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.6-4-219
+es5id: 15.2.3.6-4-89
 description: >
-    Object.defineProperty - 'O' is an Array, 'name' is an array index
-    property, test TypeError is thrown when the [[Value]] field of
-    'desc' is -0, and the [[Value]] attribute value of 'name' is +0
-    (15.4.5.1 step 4.c)
+    Object.defineProperty will throw TypeError when name.configurable
+    = false, name.writable = false, desc.value and name.value are two
+    numbers with different values (8.12.9 step 10.a.ii.1)
 includes: [propertyHelper.js]
 ---*/
 
-var arrObj = [];
 
-Object.defineProperty(arrObj, "0", {
-  value: +0
+var obj = {};
+
+Object.defineProperty(obj, "foo", {
+  value: 10,
+  writable: false,
+  configurable: false
 });
 
 try {
-  Object.defineProperty(arrObj, "0", {
-    value: -0
+  Object.defineProperty(obj, "foo", {
+    value: 20
   });
   throw new Test262Error("Expected an exception.");
 } catch (e) {
@@ -28,8 +30,8 @@ try {
   }
 }
 
-verifyProperty(arrObj, "0", {
-  value: +0,
+verifyProperty(obj, "foo", {
+  value: 10,
   writable: false,
   enumerable: false,
   configurable: false,
