@@ -2,14 +2,19 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 8.12.1-1_37
+es5id: 8.12.1-1_22
 description: >
-    Properties - [[HasOwnProperty]] (configurable, enumerable own
-    getter/setter property)
+    Properties - [[HasOwnProperty]] (literal own getter/setter
+    property)
 ---*/
 
-    var o = {};
-    Object.defineProperty(o, "foo", {get: function() {return 42;}, set: function() {;}, enumerable:true, configurable:true});
+var o = {
+  get foo() {
+    return 42;
+  },
+  set foo(x) {;
+  }
+};
 
 assert(o.hasOwnProperty("foo"), 'o.hasOwnProperty("foo") !== true');
 
